@@ -267,6 +267,32 @@ g_array_remove_index_fast (GArray* farray,
   return farray;
 }
 
+GArray*
+g_array_remove_range (GArray       *farray,
+                      guint         index_,
+                      guint         length)
+{
+  GRealArray *array = (GRealArray*) farray;
+
+  g_return_val_if_fail (array, NULL);
+  g_return_val_if_fail (index_ < array->len, NULL);
+  g_return_val_if_fail (index_ + length <= array->len, NULL);
+
+  if (index_ + length != array->len)
+    g_memmove (g_array_elt_pos (array, index_), 
+               g_array_elt_pos (array, index_ + length), 
+               (array->len - (index_ + length)) * array->elt_size);
+
+  array->len -= length;
+#ifdef ENABLE_GC_FRIENDLY
+  g_array_elt_zero (array, array->len, length);
+#else /* !ENABLE_GC_FRIENDLY */
+  g_array_zero_terminate (array);
+#endif /* ENABLE_GC_FRIENDLY */
+
+  return farray;
+}
+
 void
 g_array_sort (GArray       *farray,
 	      GCompareFunc  compare_func)
@@ -414,7 +440,7 @@ g_ptr_array_maybe_expand (GRealPtrArray *array,
 #endif /* ENABLE_GC_FRIENDLY */
       array->alloc = g_nearest_pow (array->len + len);
       array->alloc = MAX (array->alloc, MIN_ARRAY_SIZE);
-      array->pdata = g_realloc (array->pdata, sizeof(gpointer) * array->alloc);
+      array->pdata = g_realloc (array->pdata, sizeof (gpointer) * array->alloc);
 #ifdef ENABLE_GC_FRIENDLY
       for ( ; old_alloc < array->alloc; old_alloc++)
 	array->pdata [old_alloc] = NULL;
@@ -506,6 +532,28 @@ g_ptr_array_remove_index_fast (GPtrArray* farray,
   return result;
 }
 
+void
+g_ptr_array_remove_range (GPtrArray* farray,
+                          guint      index_,
+                          guint      length)
+{
+  GRealPtrArray* array = (GRealPtrArray*) farray;
+
+  g_return_if_fail (array);
+  g_return_if_fail (index_ < array->len);
+  g_return_if_fail (index_ + length <= array->len);
+
+  if (index_ + length != array->len)
+    g_memmove (&array->pdata[index_],
+               &array->pdata[index_ + length], 
+               (array->len - (index_ + length)) * sizeof (gpointer));
+
+  array->len -= length;
+#ifdef ENABLE_GC_FRIENDLY
+  g_array_elt_zero (array->pdata, array->len, length);
+#endif /* ENABLE_GC_FRIENDLY */
+}
+
 gboolean
 g_ptr_array_remove (GPtrArray* farray,
 		    gpointer data)
@@ -648,6 +696,18 @@ GByteArray* g_byte_array_remove_index_fast (GByteArray *array,
   return array;
 }
 
+GByteArray*
+g_byte_array_remove_range (GByteArray *array,
+                           guint index_,
+                           guint length)
+{
+  g_return_val_if_fail (array, FALSE);
+  g_return_val_if_fail (index_ < array->len, FALSE);
+  g_return_val_if_fail (index_ + length <= array->len, FALSE);
+
+  return (GByteArray *)g_array_remove_range ((GArray*) array, index_, length);
+}
+
 void
 g_byte_array_sort (GByteArray   *array,
 		   GCompareFunc  compare_func)
